// Common/StdOutStream.cpp

#include "StdAfx.h"

#include "StdOutStream.h"
#include "Common/IntToString.h"
#include "Common/StringConvert.h"
#include "Common/UTFConvert.h"
#include "Windows/FileDir.h"

static const char kNewLineChar =  '\n';

CStdOutStream  g_StdOut;
CStdOutStream  g_StdErr;

CStdOutStream ::CStdOutStream ()
{
	m_strLog = L"";
	m_lpCommandLine = NULL;
	m_bUnicode = 0;
	*m_lpTempSFXName = '\0';
	ReSet();
}

CStdOutStream ::~CStdOutStream ()
{
	if (m_lpCommandLine)
		free(m_lpCommandLine);
}

CStdOutStream & CStdOutStream::operator<<(CStdOutStream & (*aFunction)(CStdOutStream  &))
{
  (*aFunction)(*this);    
  return *this;
}

CStdOutStream & endl(CStdOutStream & outStream)
{
  return outStream << kNewLineChar;
}

CStdOutStream & CStdOutStream::operator<<(const char *string)
{
	m_strLog += GetUnicodeString(string);
	return *this;
}

CStdOutStream & CStdOutStream::operator<<(const wchar_t *string)
{
	m_strLog += string;
	return *this;
}

CStdOutStream & CStdOutStream::operator<<(char c)
{
	m_strLog += c;
	return *this;
}

CStdOutStream & CStdOutStream::operator<<(int number)
{
	char textString[32];
	ConvertInt64ToString(number, textString);
	return operator<<(textString);
}

CStdOutStream & CStdOutStream::operator<<(UINT64 number)
{
  char textString[32];
  ConvertInt64ToString(number, textString);
  return operator<<(textString);
}

// oϐ
void CStdOutStream::ReSet()
{
	m_pDlgProgress = NULL;
	m_hThread = NULL;
	m_strLog.Empty();
	if (m_lpCommandLine)
	{
		free(m_lpCommandLine);
		m_lpCommandLine = NULL;
	}
}

// o͂Rs[
// lpCopyBufNULL̎͏o̓obt@̃|C^̂Ԃ
void CStdOutStream::CopyBuf(LPSTR lpCopyBuf, DWORD dwBufSize)
{
	if (lpCopyBuf == NULL)
		return;

	AString strLog;
	if (m_bUnicode)
		::ConvertUnicodeToUTF8(m_strLog, strLog);
	else
		strLog = GetOemString(m_strLog);
	strLog.Replace("\n", "\r\n");
	strncpy(lpCopyBuf, strLog, dwBufSize);
	lpCopyBuf[dwBufSize - 1] = '\0';
}

// ꂽR}hR}hCɂċL
void CStdOutStream::SetCommandLine(const CSplitCmdLine& scl)
{
	UString strCmdLine = L"7-zip32.dll ";
	strCmdLine += scl.m_argv[0];
	if (scl.m_bSfx)
	{
		strCmdLine += L" \"-sfx";
		strCmdLine += GetTempSFXName();
		strCmdLine += L"\"";
	}
	if (!scl.m_bScs && !m_bUnicode)
		strCmdLine += L" -scsWIN";

	for (int i = 1; i < scl.m_argc; ++i)
	{
		strCmdLine += L" \"";
		strCmdLine += scl.m_argv[i];
		strCmdLine += L"\"";
	}
	if (m_lpCommandLine)
		free(m_lpCommandLine);
	m_lpCommandLine = _wcsdup(strCmdLine);
}

// LꂽR}hC擾
LPWSTR CStdOutStream::MyGetCommandLineW()
{
	return m_lpCommandLine;
}

// ꎞSFXt@C̎擾
LPCWSTR CStdOutStream::GetTempSFXName()
{
	if (*m_lpTempSFXName == '\0')
	{
		UString strTempPath;
		UString strTempFile;
		::NWindows::NFile::NDirectory::MyGetTempPath(strTempPath);
		::NWindows::NFile::NDirectory::MyGetTempFileName(strTempPath, L"sfx", strTempFile);
		wcsncpy(m_lpTempSFXName, strTempFile, MAX_PATH);
		m_lpTempSFXName[MAX_PATH - 1] = '\0';
	}
	return m_lpTempSFXName;
}